package com.fireworksdk.bridge.components.videofeed

import android.content.Context
import android.util.AttributeSet
import android.view.LayoutInflater
import android.widget.RelativeLayout
import androidx.constraintlayout.widget.ConstraintLayout
import com.loopnow.fireworklibrary.FeedType
import com.loopnow.fireworklibrary.models.FeedLayout
import com.loopnow.fireworklibrary.views.VideoFeedView
import com.fireworksdk.bridge.R
import com.fireworksdk.bridge.models.*
import com.fireworksdk.bridge.utils.FWVideoPlayerUtils
import com.loopnow.fireworklibrary.views.PlaylistGroupFeedView


class FWVideoFeed(
  context: Context,
  attrs: AttributeSet?
) : RelativeLayout(context, attrs) {

  private var videoFeedViewContainer: ConstraintLayout? = null
  var videoFeedView: VideoFeedView? = null

  private var playlistGroupFeedViewContainer: ConstraintLayout? = null
  var playlistGroupFeedView: PlaylistGroupFeedView? = null

  val videoFeedPropsModel: FWVideoFeedPropsModel by lazy { FWVideoFeedPropsModel() }
  private var currentMode: String? = null
  private var currentPlaylistGroupId: String? = null
  private var currentSource: String? = FWVideoFeedSource.Discover.rawValue
  private var currentChannelId: String? = null
  private var currentPlaylistId: String? = null
  private var currentDynamicContentParameters: HashMap<String, List<String>>? = null

  constructor(context: Context) : this(context, null)

  fun setProps(props: FWVideoFeedPropsModel?) {
    if (props != null) {
      videoFeedPropsModel.source = props.source
      videoFeedPropsModel.channel = props.channel
      videoFeedPropsModel.playlist = props.playlist
      videoFeedPropsModel.playlistGroup = props.playlistGroup
      videoFeedPropsModel.mode = props.mode
      videoFeedPropsModel.videoFeedConfiguration = props.videoFeedConfiguration
      videoFeedPropsModel.videoPlayerConfiguration = props.videoPlayerConfiguration
      videoFeedPropsModel.dynamicContentParameters = props.dynamicContentParameters
    }
    setData()
  }

  fun setSourceProps(source: String?) {
    videoFeedPropsModel.source = source
    setData()
  }

  fun setChannelProps(channel: String?) {
    videoFeedPropsModel.channel = channel
    setData()
  }

  fun setPlaylistProps(playlist: String?) {
    videoFeedPropsModel.playlist = playlist
    setData()
  }

  fun setPlaylistGroupProps(playlistGroup: String?) {
    videoFeedPropsModel.playlistGroup = playlistGroup
    setData()
  }

  fun setModeProps(mode: String?) {
    videoFeedPropsModel.mode = mode
    setData()
  }

  fun setDynamicContentParametersProps(dynamicContentParameters: HashMap<String, List<String>>?) {
    videoFeedPropsModel.dynamicContentParameters = dynamicContentParameters
    setData()
  }

  fun setVideoFeedConfigProps(config: FWVideoFeedConfigModel?) {
    videoFeedPropsModel.videoFeedConfiguration = config
    setData()
  }

  fun setVideoPlayerConfigProps(config: FWVideoPlayerConfigModel?) {
    videoFeedPropsModel.videoPlayerConfiguration = config
    setData()
  }

  fun refresh() {
    videoFeedView?.refreshFeed()
    playlistGroupFeedView?.refresh()
  }

  private fun setData() {
    val source = videoFeedPropsModel.source
    val mode = videoFeedPropsModel.mode
    val dynamicContentParameters = videoFeedPropsModel.dynamicContentParameters
    val channel = videoFeedPropsModel.channel
    val playlist = videoFeedPropsModel.playlist
    val playlistGroup = videoFeedPropsModel.playlistGroup
    val videoFeedConfiguration = videoFeedPropsModel.videoFeedConfiguration
    val videoPlayerConfiguration = videoFeedPropsModel.videoPlayerConfiguration

    if (!source.isNullOrBlank()) {
      when (source) {
        FWVideoFeedSource.PlaylistGroup.rawValue -> {
          setPlaylistGroupFeed(mode, playlistGroup, videoFeedConfiguration, videoPlayerConfiguration)
        }
        else -> {
          setFeed(mode, channel, playlist, source, dynamicContentParameters, videoFeedConfiguration, videoPlayerConfiguration)
        }
      }
    }

    currentSource = source ?: FWVideoFeedSource.Discover.rawValue
    currentMode = mode
    currentDynamicContentParameters = dynamicContentParameters
    currentChannelId = channel
    currentPlaylistId = playlist
    currentPlaylistGroupId = playlistGroup
  }

  private fun setFeed(
    mode: String?,
    channelId: String?,
    playlistId: String?,
    source: String?,
    dynamicContentParameters: HashMap<String, List<String>>?,
    videoFeedConfiguration: FWVideoFeedConfigModel?,
    videoPlayerConfiguration: FWVideoPlayerConfigModel?,
  ) {
    // add view
    addVideoFeedView()

    // set layout
    if (!mode.isNullOrBlank() && mode != currentMode) {
      val feedLayout = when (mode) {
          FWVideoFeedModel.Column.rawValue -> FeedLayout.VERTICAL
          FWVideoFeedModel.Grid.rawValue -> FeedLayout.GRID
          else -> FeedLayout.HORIZONTAL
      }

      videoFeedView?.setLayout(feedLayout)
    }

    if (!source.isNullOrBlank() && source != currentSource) {

      when (source) {
        FWVideoFeedSource.DynamicContent.rawValue -> {
          if (!channelId.isNullOrBlank()
            && (channelId != currentChannelId || differentDynamicContentParameters(currentDynamicContentParameters, dynamicContentParameters))
          ) {
            videoFeedView?.setDynamicContent(channelId, dynamicContentParameters?:hashMapOf())
          }
        }
        FWVideoFeedSource.Channel.rawValue -> {
          if (!channelId.isNullOrBlank() && channelId != currentChannelId) {
            videoFeedView?.setFeed(channelId, null, FeedType.CHANNEL)
          }
        }
        FWVideoFeedSource.Playlist.rawValue -> {
          if (!channelId.isNullOrBlank() && !playlistId.isNullOrBlank()
            && (channelId != currentChannelId || playlistId != currentPlaylistId)
          ) {
            videoFeedView?.setFeed(channelId, playlistId, FeedType.PLAYLIST)
          }
        }
        else -> {
          videoFeedView?.setFeed(null, null, FeedType.DISCOVER)
        }
      }
    }

    // set Feed Configuration
    FWVideoPlayerUtils.setVideoFeedConfig(context, videoFeedView, videoFeedConfiguration)
    // set Player Configuration
    FWVideoPlayerUtils.setVideoPlayerConfig(videoPlayerConfiguration)
  }

  private fun differentDynamicContentParameters(old: HashMap<String, List<String>>?, new: HashMap<String, List<String>>?): Boolean {
    if (old == null || new == null) {
      return old == new
    }
    if (old.size != new.size) {
      return false
    }
    if (old.keys.sorted().toSet() != new.keys.sorted().toSet()) {
      return false
    }
    val differentValues = old.keys.filter { key ->
      val oldValue = old[key] ?: emptyList()
      val newValue = new[key] ?: emptyList()
      oldValue.sorted().toSet() != newValue.sorted().toSet()
    }
    return differentValues.isEmpty()
  }

  private fun setPlaylistGroupFeed(
    mode: String?,
    playlistGroupId: String?,
    videoFeedConfiguration: FWVideoFeedConfigModel?,
    videoPlayerConfiguration: FWVideoPlayerConfigModel?,
  ) {
    // add view
    addPlaylistGroupFeedView()

    // set layout
    if (!mode.isNullOrBlank() && mode != currentMode) {
      val feedLayout = when (mode) {
        FWVideoFeedModel.Column.rawValue -> FeedLayout.VERTICAL
        FWVideoFeedModel.Grid.rawValue -> FeedLayout.GRID
        else -> FeedLayout.HORIZONTAL
      }

      playlistGroupFeedView?.setLayout(feedLayout)
    }

    // set playlistGroupId
    if (!playlistGroupId.isNullOrBlank() && playlistGroupId != currentPlaylistGroupId) {
      playlistGroupFeedView?.setPlaylistGroupId(playlistGroupId)
    }

    // set Feed Configuration
    FWVideoPlayerUtils.setPlaylistGroupFeedConfig(context, playlistGroupFeedView, videoFeedConfiguration)
    // set Player Configuration
    FWVideoPlayerUtils.setVideoPlayerConfig(videoPlayerConfiguration)
  }

  private fun addVideoFeedView() {
    removePlaylistGroupFeedView()
    if (videoFeedView != null) {
      return
    }
    videoFeedViewContainer =
      LayoutInflater.from(context).inflate(R.layout.fw_bridge_fragment_videofeed, this, false) as ConstraintLayout?
    videoFeedView = videoFeedViewContainer?.findViewById(R.id.fw_bridge_videofeed)
    if (videoFeedView != null) {
      addView(videoFeedViewContainer)
    }
  }

  private fun removeVideoFeedView() {
    if (videoFeedViewContainer == null) {
      return
    }
    removeView(videoFeedViewContainer)
    videoFeedViewContainer = null
    videoFeedView = null
  }

  private fun addPlaylistGroupFeedView() {
    removeVideoFeedView()
    if (playlistGroupFeedView != null) {
      return
    }
    playlistGroupFeedViewContainer =
      LayoutInflater.from(context).inflate(R.layout.fw_bridge_fragment_playlistfeed, this, false) as ConstraintLayout?
    playlistGroupFeedView = playlistGroupFeedViewContainer?.findViewById(R.id.fw_bridge_playlistfeed)
    if (playlistGroupFeedView != null) {
      addView(playlistGroupFeedViewContainer)
    }
  }

  private fun removePlaylistGroupFeedView() {
    if (playlistGroupFeedViewContainer == null) {
      return
    }
    removeView(playlistGroupFeedViewContainer)
    playlistGroupFeedViewContainer = null
    playlistGroupFeedView = null
  }
}
