package com.fireworksdk.bridge.flutter.platformview

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.view.View
import androidx.fragment.app.FragmentActivity
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.fireworksdk.bridge.R
import com.fireworksdk.bridge.flutter.handler.FWSdkModuleHandler
import com.fireworksdk.bridge.flutter.handler.FWStoryBlockHandler
import com.fireworksdk.bridge.flutter.models.FWChannelName
import com.fireworksdk.bridge.models.FWVideoFeedPropsModel
import com.fireworksdk.bridge.flutter.utils.FWEventUtils
import com.fireworksdk.bridge.models.FWVideoFeedSource
import com.fireworksdk.bridge.utils.FWGsonUtil
import com.fireworksdk.bridge.utils.FWLogUtils
import com.loopnow.fireworklibrary.FeedType
import com.loopnow.fireworklibrary.FwSDK
import com.loopnow.fireworklibrary.models.VideoContentStatus
import com.loopnow.fireworklibrary.views.FireworkPlayerFragment
import com.loopnow.fireworklibrary.views.OnVideoContentStatusListener
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.platform.PlatformView

class FWStoryBlockPlatformView(
    private val activity: FragmentActivity,
    private val id: Int,
    creationParams: Map<String?, Any?>?,
    binaryMessenger : BinaryMessenger,
): PlatformView {

    private var broadcastReceiver: BroadcastReceiver = MyReceiver()
    private val fireworkPlayerFragment = FireworkPlayerFragment()

    private var videoFeedPropsModel: FWVideoFeedPropsModel? = null

    private val methodChannel = MethodChannel(
        binaryMessenger,
        FWChannelName.FireworkStoryBlock.rawValue.replace("(viewId)", "$id")
    )

    init {
        val filter = IntentFilter()
        filter.addAction(FW_STORY_BLOCK_INIT_ACTION)
        LocalBroadcastManager.getInstance(activity).registerReceiver(broadcastReceiver, filter)

        methodChannel.setMethodCallHandler(FWStoryBlockHandler())

        val videoFeedProps: FWVideoFeedPropsModel? = FWGsonUtil.fromObject(creationParams, FWVideoFeedPropsModel::class.java)
        videoFeedPropsModel = videoFeedProps
        addStoryBlockListener()
    }

    private fun addStoryBlockListener() {

        FwSDK.addOnPlayerVideoContentStatusListener(object: OnVideoContentStatusListener {
            override fun currentStatus(status: VideoContentStatus, extra: String) {
                when (status) {
                    VideoContentStatus.LoadingContent -> {
                        FWLogUtils.d { "FWStoryBlockPlatformView VideoContentStatus.LoadingContent" }
                    }
                    VideoContentStatus.LoadingContentFailed -> {
                        FWLogUtils.d { "FWStoryBlockPlatformView VideoContentStatus.LoadingContentFailed" }
                        FWEventUtils.sendStoryBlockLoadFailedEvent(methodChannel, VideoContentStatus.LoadingContentFailed.rawValue, extra)
                    }
                    VideoContentStatus.ContentLoaded -> {
                        FWLogUtils.d { "FWStoryBlockPlatformView VideoContentStatus.ContentLoaded" }
                        FWEventUtils.sendStoryBlockLoadSuccessEvent(methodChannel)
                    }
                    VideoContentStatus.ContentOver -> {
                        FWLogUtils.d { "FWStoryBlockPlatformView VideoContentStatus.ContentOver" }
                    }
                    VideoContentStatus.ContentExpired -> {
                        FWLogUtils.d { "FWStoryBlockPlatformView VideoContentStatus.ContentExpired" }
                    }
                    else -> {}
                }
            }
        })
    }

    override fun getView(): View {
        val view = activity.layoutInflater.inflate(R.layout.fw_bridge_fragment_storyblock, null)
        view.addOnAttachStateChangeListener(object : View.OnAttachStateChangeListener {
            override fun onViewAttachedToWindow(v: View?) {
                val fm = activity.supportFragmentManager
                val ft = fm.beginTransaction()
                ft.replace(R.id.fragment_host, fireworkPlayerFragment)
                ft.commitNow()

                setStoryBlockParameters()
            }

            override fun onViewDetachedFromWindow(v: View?) {
            }

        })
        return view
    }

    override fun dispose() {
        FWLogUtils.d { "FWStoryBlockPlatformView dispose viewid:${id}, source: ${videoFeedPropsModel?.source}, initSuccess: ${FWSdkModuleHandler.initSuccess}" }
        LocalBroadcastManager.getInstance(activity).unregisterReceiver(broadcastReceiver)
    }

    private fun setStoryBlockParameters() {
        FWLogUtils.d { "FWStoryBlockPlatformView setStoryBlockParameters viewid:${id}, source: ${videoFeedPropsModel?.source}, initSuccess: ${FWSdkModuleHandler.initSuccess}" }
        val source = videoFeedPropsModel?.source
        if (source.isNullOrBlank()) {
            return
        }

        val dynamicContentParameters = videoFeedPropsModel?.dynamicContentParameters
        val channel = videoFeedPropsModel?.channel
        val playlist = videoFeedPropsModel?.playlist

        if (FWSdkModuleHandler.initSuccess) {
            when (source) {
                FWVideoFeedSource.DynamicContent.rawValue -> {
                    fireworkPlayerFragment.setStoryBlockParameters(FeedType.DYNAMIC_CONTENT, channel, playlist, dynamicContentParameters)
                }
                FWVideoFeedSource.Playlist.rawValue -> {
                    fireworkPlayerFragment.setStoryBlockParameters(FeedType.PLAYLIST, channel, playlist)
                }
                FWVideoFeedSource.Channel.rawValue -> {
                    fireworkPlayerFragment.setStoryBlockParameters(FeedType.CHANNEL, channel)
                }
                else -> {
                    fireworkPlayerFragment.setStoryBlockParameters(FeedType.DISCOVER)
                }
            }
        }
    }

    companion object {
        const val FW_STORY_BLOCK_INIT_ACTION = "FW_STORY_BLOCK_INIT_ACTION"
        const val FW_STORY_BLOCK_INIT_INTENT = "FW_STORY_BLOCK_INIT_INTENT"
    }

    inner class MyReceiver : BroadcastReceiver() {

        override fun onReceive(context: Context?, intent: Intent) {
            val initSuccess = intent.getBooleanExtra(FW_STORY_BLOCK_INIT_INTENT, false)
            if (initSuccess) {
                setStoryBlockParameters()
            }
        }
    }
}