import 'dart:async';

import 'package:flutter/material.dart';
import '../../utils/model_utils/story_block_source_util.dart';
import 'story_block_platform_view.dart';

import '../../constants/fw_event_name.dart';
import '../../firework_sdk.dart';
import '../../fw_event_bus.dart';
import '../../models/ad_badge_configuration.dart';
import '../../models/fw_error.dart';
import '../../models/story_block_source.dart';
import '../../utils/model_utils/ad_badge_configuration_util.dart';

typedef StoryBlockLoadFinishedCallback = void Function(FWError? error);

// The StoryBlock widget is only supported on iOS.
class StoryBlock extends StatefulWidget {
  /// Indicate whether to use hybrid composition on the Android side.
  ///
  /// Sets this to true to use hybrid composition on the Android side.
  /// Sets this to false to use virtual display on the Android side.
  /// Defaults to use hybrid composition.
  static bool useHybridCompositionForAndroid = true;

  /// The source of the story block.
  final StoryBlockSource source;

  /// The channel id of the story block.
  ///
  /// Required when the source is set as `channel` or `playlist` or `dynamicContent`.
  final String? channel;

  /// The playlist id of the story block.
  ///
  /// Required when the source is set as `playlist`.
  final String? playlist;

  /// The dynamic content parameters of the story block.
  ///
  /// Required when the source is set as `dynamicContent`.
  final Map<String, List<String>>? dynamicContentParameters;

  /// Specifies if Picture in Picture is enabled. Only supported on iOS.
  final bool? enablePictureInPicture;

  /// The width of the story block.
  final double? width;

  /// The height of the story block.
  final double? height;

  /// The callback of the feed loading result.
  ///
  /// It means loading successfully when error equals to null.
  final StoryBlockLoadFinishedCallback? onStoryBlockLoadFinished;

  const StoryBlock({
    Key? key,
    required this.source,
    this.channel,
    this.playlist,
    this.dynamicContentParameters,
    this.width,
    this.height,
    this.enablePictureInPicture,
    this.onStoryBlockLoadFinished,
  }) : super(key: key);

  @override
  State<StoryBlock> createState() => _StoryBlockState();
}

class _StoryBlockState extends State<StoryBlock> {
  StreamSubscription? _eventSubscription;

  @override
  void initState() {
    super.initState();
    _eventSubscription = FWEventBus.getInstance().on().listen((event) {
      if (event.eventName == FWEventName.adBadgeConfigurationUpdated) {
        setState(() {});
      } else if (event.eventName == FWEventName.shareBaseURLUpdated) {
        setState(() {});
      }
    });
  }

  @override
  void dispose() {
    _eventSubscription?.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return SizedBox(
      width: widget.width,
      height: widget.height,
      child: StoryBlockPlatformView(
        key: ValueKey(_generateKey()),
        source: widget.source,
        channel: widget.channel,
        playlist: widget.playlist,
        dynamicContentParameters: widget.dynamicContentParameters,
        enablePictureInPicture: widget.enablePictureInPicture,
        useHybridCompositionForAndroid:
            StoryBlock.useHybridCompositionForAndroid,
        onStoryBlockPlatformViewLoadFinished: widget.onStoryBlockLoadFinished,
      ),
    );
  }

  String _generateKey() {
    final source = StoryBlockSourceUtil.convertToRawFeedSource(widget.source);
    final channel = widget.channel ?? "";
    final playlist = widget.playlist ?? "";
    final useHybridCompositionForAndroid =
        StoryBlock.useHybridCompositionForAndroid;
    final shareBaseURL = FireworkSDK.getInstance().shareBaseURL ?? "";
    final adBadgeConfiguration =
        FireworkSDK.getInstance().adBadgeConfiguration ??
            AdBadgeConfiguration();
    final adBadgeTextType =
        AdBadgeConfigurationUtil.convertBadgeTextTypeToRawType(
            adBadgeConfiguration.badgeTextType);
    final backgroundColorOfAdBadge = adBadgeConfiguration.backgroundColor ?? '';
    final textColorOfAdBadge = adBadgeConfiguration.textColor ?? '';
    final dynamicContentParametersString =
        _generateDynamicContentParametersString();
    final enablePictureInPicture = widget.enablePictureInPicture ?? false;

    String key = "source:$source";
    key += "_channel:$channel";
    key += "_playlist:$playlist";
    key += "_useHybridCompositionForAndroid:$useHybridCompositionForAndroid";
    key += "_shareBaseURL:$shareBaseURL";
    key += "_adBadgeTextType:$adBadgeTextType";
    key += "_backgroundColorOfAdBadge:$backgroundColorOfAdBadge";
    key += "_textColorOfAdBadge:$textColorOfAdBadge";
    key += "_dynamicContentParameters:$dynamicContentParametersString";
    key += "_enablePictureInPicture:$enablePictureInPicture";

    return key;
  }

  String _generateDynamicContentParametersString() {
    final parameters = widget.dynamicContentParameters;
    if (parameters == null) {
      return "";
    }

    var resultString = "";
    final sortedKeyList = parameters.keys.toList()..sort();
    for (final key in sortedKeyList) {
      final value = parameters[key];
      if (value != null) {
        final valueString = value.join(",");
        if (resultString.isNotEmpty) {
          resultString += "_";
        }
        resultString += "$key:$valueString";
      }
    }

    return resultString;
  }
}
